package util;

import java.awt.*;
import javax.swing.*;

/**
 * This object deals both decoding the response from a K2 "DS" command and
 * for displaying it on-screen.  There is also a method, forceDisplay(), that
 * is useful to force this object to put something on the display other than
 * data from a DS command.
 *
 * From the KIO2 manual, the response to a "DS;" command is:
 *
 * GET format: DS; (no data). Returns everything needed to reproduce
 * the information shown on the K2 LCD. The format of K2's response
 * message is: DSttttttttaf; where <tttttttt> is the LCD text and
 * decimal point data, <a> is annunciator data, and <f> is annunciator
 * flash data.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br>
 * March 2007
 */

public class DSdecoder {

	private String forceString = null;
	private boolean tens, hundreds; // Are these digits (Hz) used?
	private boolean[] an; // Annunciator on/off data.
	private boolean[] dp; // Decimal point on/off data.
	private boolean[] flash; // Annunciator flashing/not flasing data.
	private char[] ch; // Character to display;

	/** Create the decoder as well as set of 7 segment digit displays. */
	public DSdecoder() {
		an = new boolean[8];
		dp = new boolean[8];
		flash = new boolean[8];
		ch = new char[8];
	}

	/** Is K2's finest tuning rate currently set to 10 Hz steps? */
	public boolean areTensUsed() { return tens; }
	/** Is K2's finest tuning rate currently set to 100 Hz steps? */
	public boolean areHundredsUsed() { return hundreds; }

	/** Force a given string, s, to be displayed.  Be sure that your
	 * requested letters can in fact be displayed in 7 segments.  The
	 * decimal points are turned off and annunciators anBegin through anEnd
	 * are turned on. */
	public void forceDisplay(String s, int anBegin, int anEnd) {

		forceString = s;
		if (s == null)
			return;

		for (int i = 0; i < 8; i++) {

			// Set character.
			if (i < s.length())
				ch[i] = s.charAt(i);
			else
				ch[i] = '@'; // K2's char for a space.

			// Set annunciator.
			if (i >= anBegin && i < anEnd)
				an[i] = true;
			else
				an[i] = false;

			// Set decimal point.
			dp[i] = false;
		}
	}

	public String getDisplay(byte[] ds) {

		String display = "";
		for (int i = 2; i < 10; i++) {
			boolean dp = ((ds[i] & 0x80) != 0);
			if (dp)
				display += ".";
			char b = (char) (ds[i] & 0x7f); // Clear MSBit.
			switch (b) {
			case '<': display += 'L'; break;
			case '>': display += '-'; break;
			case '@': display += ' '; break;
			case 'K': display += 'H'; break;
			case 'M': display += 'N'; break;
			case 'Q': display += 'O'; break;
			case 'V': display += 'U'; break;
			case 'W': display += 'I'; break;
			case 'X': display += 'C'; break;
			case 'Z': display += 'C'; break;
			case '[': display += 'R'; break;
			default: display += b; break;
			}
		}
		return display.trim();

	}

	public void forceDisplayScroll(int rightPos, char newChar) {
		int maxIndex = (rightPos > 4) ?  maxIndex = 4 : rightPos;
		for (int i = 0; i < maxIndex; i++)
			ch[i] = ch[i+1];
		ch[rightPos] = newChar;
	}

	/** Display on the virtual K2 screen the most recently received DS
	 * response data (or what has been forced to display by forceDisplay()).
	 */
//	public void display(Graphics g, JLabel jp, byte[] DSbytes, K2.Mode state) {
//
//		if (DSbytes == null)
//			return; // Donta wastse-a my time.
//
//		// Check DSbytes is a valid return string from a 'DS' cmd.
//		if (forceString == null
//			&& (DSbytes[0] & 0x7f) == 'D' && (DSbytes[1] & 0x7f) == 'S') {
//
//			//// Turn everything on to check positioning.
//			//	DSbytes = new byte[12];
//			//	for (int i = 0; i < 12; i++)
//			//		DSbytes[i] = '8' | (byte) 0x80;
//			//	DSbytes[10] = (byte) 0x7f;
//
//			// Decode response from DS command.
//			setDisplayString(DSbytes);
//			if (state == K2.Mode.NORMAL) {
//				setAnnunciators(DSbytes[10]);
//				setAnnunciatorFlash(DSbytes[11]);
//			} else if (state == K2.Mode.DSP || state == K2.Mode.POWER) {
//				for (int i = 0; i < an.length; i++)
//					an[i] = false;
//			} else if (state == K2.Mode.MENU) {
//				// Compensate for K2 not giving annunciator info for MENU.
//				an[0] = an[1] = an[2] = true;
//				boolean anState = true;
//				for (int i = 3; i < ch.length; i++) {
//					if (anState && ch[i] == '@')
//						anState = false;
//					an[i] = anState;
//				}
//			} else if (state == K2.Mode.EDIT) {
//				// Compensate for K2 not giving annunciator info for EDIT.
//				boolean anState = false;
//				an[0] = an[1] = an[2] = an[3] = false;
//				for (int i = 4; i < ch.length; i++) {
//					if (!anState && ch[i] != '@')
//						anState = true;
//					an[i] = anState;
//				}
//			}
//		}
//
//		// Display response;
//		for (int i = 0; i < 8; i++)
//			digit[i].display(g, jp, ch[i], dp[i], an[i]);
//
//	}

	/** Use DS response bytes to set each 7-seg lcd digit. */
	private void setDisplayString(byte[] DSbytes) {

		// Convert due to Java's lack of unsigned ints.
		int[] dsInts = Unsigned.bytes2UInts(DSbytes);

		// Get lcd display string.
		for (int i = 2; i < 10; i++) {

			if ((dsInts[i] & 0x80) == 0x80)
				// MSB set means we must display decimal point.
				dp[(i+5)%8] = true;
			else
				dp[(i+5)%8] = false;

			// Strip off MSB.
			char displayChar = (char) (dsInts[i] & 0x7f);
			ch[i-2] = displayChar;

			if (i == 7)
				hundreds = (displayChar == '@') ? false : true;
			else if (i == 8) {
				tens = (displayChar == '@') ? false : true;
				if (tens)
					hundreds = false;
			}
		}
	}

	/**
	 * Set which annunciators are on and which are off.
	 *
	 * <p>Annunciator data: This field is a single byte whose value is between
	 * 0x80 and 0xFF. Bit 7 is always 1. The other 7 bits indicate the
	 * on/off states of the 8 annunciators. Only 7 bits are needed because
	 * the VFO A state is always the opposite of VFO B state. The bits
	 * are defined as follows:
     *
	 * B7: Always 1<br>
	 * B3: 1=ATT on<br>
	 * B6: 1=NB on<br>
	 * B2: 0=VFO A selected, 1=VFO B selected<br>
	 * B5: 1=ANT2 selected<br>
	 * B1: 1=RIT on<br>
	 * B4: 1=PREAMP on<br>
	 * B0: 1=XIT on
	 */

	private void setAnnunciators(byte annunciators) {
		an[0]       = ((annunciators & 0x40) != 0);
		an[1]       = ((annunciators & 0x20) != 0);
		an[2]       = ((annunciators & 0x10) != 0);
		an[3]       = ((annunciators & 0x08) != 0);
		an[5]       = ((annunciators & 0x04) != 0);
		an[4]       = !an[5];
		an[6]       = ((annunciators & 0x02) != 0);
		an[7]       = ((annunciators & 0x01) != 0);
	}

	/**
	 * Set which annunciators are flashing and which aren't.
	 *
	 * <p>Annunciator flash data: This field is a single byte whose value is
	 * between 0x80 and 0xFF. Bit 7 is always 1. The other 7 bits indicate
	 * the flash/non-flashed states of the 8 annunciators, providing useful
	 * status information (such as whether the transceiver is operating
	 * in SPLIT mode). The bits are defined as follows:
     * 
	 * B7: Always 1<br>
	 * B6: 1=NB flashing (LO THR)<br>
	 * B5: 1=ANT2 flashing (not used)<br>
	 * B4: 1=PRE flashing (not used)<br>
	 * B3: 1=ATT flashing (not used)<br>
	 * B2: 1=active VFO flashing (SPLIT mode)<br>
	 * B1: 1=RIT flashing (RIT/XIT range > minimum)<br>
	 * B0: 1=XIT flashing (RIT/XIT range > minimum)
	 */

	private void setAnnunciatorFlash(byte flashByte) {
		flash[0] = ((flashByte & 0x40) != 0) ? true : false;
		flash[1] = flash[2] = flash[3] = false;
		flash[4] = ((flashByte & 0x04) != 0) ? true : false;
		flash[5] = ((flashByte & 0x02) != 0) ? true : false;
		flash[6] = ((flashByte & 0x01) != 0) ? true : false;
	}

	/** "Just the bytes, ma'am." */
	private byte[] s2b(String s) { return s.getBytes(); }

}
